using System;
using System.Collections.Generic;
using System.Text;

namespace PrototypeFactory
{
    public abstract class AbstractFactory : IAbstractFactory
    {
        public T Create<T>()
            where T : ICloneable
        {
            IFactoryMethod<T> factoryMethod = this as IFactoryMethod<T>;
            if (factoryMethod != null)
            {
                factoryMethod.Prototype.Clone();
            }
            // throw new ArgumentException();
            return default(T);
        }

        public bool IsProduct<T>()
            where T : ICloneable
        {
            return this is IFactoryMethod<T>;
        }

        public void SetPrototype<T>(T prototype)
            where T : ICloneable
        {
            IFactoryMethod<T> factoryMethod = this as IFactoryMethod<T>;
            if (factoryMethod != null)
            {
                factoryMethod.Prototype = prototype;
            }
            // throw new ArgumentException();
        }

        public T GetPrototype<T>()
            where T : ICloneable
        {
            IFactoryMethod<T> factoryMethod = this as IFactoryMethod<T>;
            if (factoryMethod != null)
            {
                return factoryMethod.Prototype;
            }
            // throw new ArgumentException();
            return default(T);
        }
    }
}
